using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;

namespace Parallax.Serial.IO
{
	//=========================================================================
	/// <summary>
	/// Represents the parallax basic stamp based hardware. Use this object to 
	/// create an interface to the robot to send and receive commands.
	/// </summary>
	public class Robot
	{
		//=========================================================================
		#region -= declarations =-

		//---- internal bits
		System.IO.Ports.SerialPort m_prtSerial = new SerialPort();
		

		//---- delegates
		public delegate void RobotDataReceivedEventHandler(Robot robot, SerialDataReceivedEventArgs e);
		//---- events
		public event RobotDataReceivedEventHandler DataReceived;


		#endregion
		//=========================================================================

		//=========================================================================
		#region -= properties =-

		/// <summary>
		/// Provides access to the underlying port. eventually i want to encapsulate 
		/// this better and insulate consumers from it, but for now it's here.
		/// </summary>
		public SerialPort Port
		{
			get { return this.m_prtSerial; }
		}

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= constructors =-

		public Robot()
		{
			Initialize();
		}

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= private methods =-

		//=========================================================================
		/// <summary>
		/// Initializes a serial connection via COM1 to the robot. (9600, 8, 1, N)
		/// </summary>
		private void Initialize()
		{
			//---- set the com port we are using from the PC
			m_prtSerial.PortName = "COM1";

			//---- set the baud rate (the rate that data is sent and received in bits/sec)
			m_prtSerial.BaudRate = 9600;

			//---- parity bit for error detection purposes: even, odd, or none
			m_prtSerial.Parity = Parity.None;

			//---- set stop bit to one (used to separate the data into frames)
			m_prtSerial.StopBits = StopBits.One;
			
			//---- set number of bits in each frame of data
			m_prtSerial.DataBits = 8;

			//---- no handshaking used
			m_prtSerial.Handshake = Handshake.None;

			//---- wire up the data received event
			m_prtSerial.DataReceived += new SerialDataReceivedEventHandler(m_prtSerial_DataReceived);
		}
		//=========================================================================

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= public methods =-

		//=========================================================================
		/// <summary>
		/// 
		/// </summary>
		/// <remarks>Opens a port if it is not open and does not close it. If you use 
		/// this method, make sure to call Close or dispose the object when you're 
		/// finished.</remarks>
		/// <param name="i"></param>
		public void SetNumberOfBlinks(int i)
		{
			//---- if the port isn't open, open it
			if (!this.m_prtSerial.IsOpen)
			{ this.m_prtSerial.Open(); }

			//---- send the string over the serial port
			// note: writeline adds a newline character to the end which tells 
			// the chip when the string is terminated.
			m_prtSerial.WriteLine(i.ToString());
			
			
			//---- close the serial comm port
			//this.m_prtSerial.Close();
		}
		//=========================================================================

		//=========================================================================
		/// <summary>
		/// Opens the underlying port connection to the robot. If you are waiting 
		/// for recieved data, you must keep the port open. However, you should 
		/// close the port when you're finished. 
		/// </summary>
		public void Open()
		{
			//---- if the port isn't open, open it
			if (!this.m_prtSerial.IsOpen)
			{ this.m_prtSerial.Open(); }
	
		}
		//=========================================================================

		//=========================================================================
		/// <summary>
		/// Closes the underlying port connection to the robot. If you are waiting 
		/// for recieved data, you must keep the port open. However, you should 
		/// close the port when you're finished. 
		/// </summary>
		public void Close()
		{
			//---- close the port if it's open
			if (this.m_prtSerial.IsOpen)
			{ this.m_prtSerial.Close(); }
		
		}
		//=========================================================================

		//=========================================================================
		/// <summary>
		/// Closes the underlying port connection
		/// </summary>
		public void Dispose()
		{
			//---- close the port
			this.Close();
		}
		//=========================================================================

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= event handlers =-

		//=========================================================================
		void m_prtSerial_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
			if (this.DataReceived != null)
			{
				this.DataReceived(this, e);
			}
		}
		//=========================================================================

		#endregion
		//=========================================================================

	}
	//=========================================================================

}
